@@ -1,5 +1,5 @@
 package Mongoose;
-$Mongoose::VERSION = '2.00';
+$Mongoose::VERSION = '2.01';
 use MooseX::Singleton;
 use Class::MOP;
 use MongoDB;
@@ -40,13 +40,25 @@ has _config => ( # Store document classes configuration
     default => sub{{}}
 );
 
+has _ns => ( # Selected namespace/tenant
+    is      => 'rw',
+    isa     => 'Str',
+    default => sub{'default'}
+);
+
+sub namespace {
+    my $self = shift;
+    if ( my $name = shift ) { $self->_ns($name) }
+    $self->_ns;
+}
+
 sub db {
     my $self = shift;
     my %p    = @_ == 1 ? (db_name=>shift) : @_;
-    my $now  = delete $p{'-now'};
+    my $now  = delete($p{'-now'}) || defined wantarray;
     my $name = $self->_add_args( \%p );
 
-    return $self->connect($name) if $now || defined wantarray;
+    return $self->connect($name) if $now;
 }
 
 sub class_config {
@@ -66,12 +78,18 @@ sub class_config {
 sub _add_args {
     my ( $self, $args ) = @_;
     my $name = 'default';
+    my $ns   = delete $args->{namespace} || $self->_ns;
+    $ns      = [$ns] unless ref $ns eq 'ARRAY';
+
     if ( my $class = delete $args->{class} ) {
         $class = [$class] unless ref $class eq 'ARRAY';
         $name  = join "-", @$class;
         $self->_args->{class}{$_} = $name for @$class;
     }
-    $self->_args->{db}{$name} = $args;
+
+    # Keep track of config for every namespace
+    $self->_args->{db}{$_}{$name} = $args for @$ns;
+
     $name;
 }
 
@@ -79,33 +97,41 @@ sub _add_args {
 sub _name_for_class {
     my ( $self, $class ) = @_;
     return 'default' unless $class;
-    $self->_args->{class}{$class} || 'default';
+    $self->_args->{class}{$self->aliased($class)} || 'default';
 }
 
 # Go thru class-db mapping and ensure to get a connected db.
-sub _db_for_class {
+sub connection {
     my ( $self, $class ) = @_;
     my $name = $self->_name_for_class($class);
-    $self->_db->{$name} || $self->connect($name);
+    $self->_db->{$self->_ns}{$name} || $self->connect($name);
 }
 
 sub connect {
     my ( $self, $name ) = @_;
     $name ||= 'default';
-    my %p   = %{ $self->_args->{db}{$name} };
-    my $data_db_name = $p{db_name};
+    my $ns  = $self->_ns;
 
-    $self->_client->{$name} = MongoDB::MongoClient->new(%p)
-      unless ref $self->_client->{$name};
+    confess "Namespace `$ns` is not defined" unless $self->_args->{db}{$ns};
 
-    $self->_db->{$name} = $self->_client->{$name}->get_database( $data_db_name );
+    # Ensure we have a config for $ns and $name or fallback to defaults
+    unless ( exists $self->_args->{db}{$ns}{$name} ) {
+        if    ( exists $self->_args->{db}{$ns}{default} ) { $name = 'default' }
+        elsif ( exists $self->_args->{db}{default}{$name} ) { $ns = 'default' }
+        else { ($ns, $name) = ('default', 'default')  }
+    }
+
+    my %conf    = %{ $self->_args->{db}{$ns}{$name} };
+    my $db_name = delete $conf{db_name};
+    my $client  = $self->_get_client(%conf);
+
+    $self->_db->{$ns}{$name} = $client->get_database( $db_name );
 }
 
-sub connection {
-    my ( $self, $name ) = @_;
-    $name ||= 'default';
-    $self->_client->{$name} and return $self->_client->{$name};
-    $self->connect($name) and return $self->_client->{$name};
+sub _get_client {
+    my ( $self, %conf ) = @_;
+    $conf{host} ||= 'mongodb://localhost:27017';
+    $self->_client->{$conf{host}} ||= MongoDB::MongoClient->new(%conf);
 }
 
 sub load_schema {
@@ -127,6 +153,15 @@ sub load_schema {
             $self->aliased($short_name => $module);
         }
     }
+
+    # Resolve class names on configured database per loaded class
+    if ( $shorten && ( my $class_map = $self->_args->{class} ) ) {
+        for ( keys %$class_map ) {
+            $class_map->{$self->aliased($_)} = delete $class_map->{$_};
+        }
+    }
+
+    $self;
 }
 
 sub aliased {
@@ -221,7 +256,7 @@ use one of these options:
 
     Mongoose->db( db_name=>'mydb', -now=>1 );  # connect now
 
-    # or by wating for a return value
+    # or by waiting for a return value
 
     my $db = Mongoose->db( 'mydb' );
 
@@ -246,10 +281,30 @@ by calling db() several times:
         class   => [qw/ Author Post /]
     );
 
+There is one more level of abstraction called C<namespace> so you can implement multitenant schemas,
+with that you can map different database configuration to your clases and your schema will select
+the ones corresponding to the current C<namespace>. In most of the use cases it will just defalt to
+the "default" namespace.
+
+    # Default host/database for all loaded classes
+    Mongoose->db( 'mydb' );
+
+    # Other database for some classes on a different namespace
+    Mongoose->db(
+        db_name   => 'other_db',
+        class     => [qw/ Category Post /],
+        namespace => 'blog_b'
+    );
+
 =head2 connect
 
 Connects to Mongo using the connection arguments passed to the C<db> method.
 
+=head2 connection
+
+Returns a connection to the database for the provided class name or the default
+connection when no class name is provided.
+
 =head2 load_schema
 
 Uses L<Module::Pluggable> to C<require> all modules under a given search path
@@ -278,15 +333,18 @@ Will shorten the module name to it's last bit:
 
     Author->new( ... );
 
-=head2 connection
-
-Sets/returns the current connection object, of class L<MongoDB::MongoClient>.
-
-Defaults to whatever MongoDB defaults.
-
 =head2 disconnect
 
-Unsets the Mongoose connection handler.
+Unsets the Mongoose connection handler/s.
+
+=head2 namespace
+
+The current namespace. You will use this in case your schema db's are configured using namespaces as described in C<db>.
+You can switch the namespace by setting it like:
+
+   Mongoose->namespace('my_namespace');
+
+It defauls to the "default" namespace.
 
 =head2 class_config
 
@@ -303,7 +361,7 @@ by replacing double-colon C<::> with underscores C<_>, separating camel-case,
 such as C<aB> with C<a_b> and uppercase with lowercase letters.
 
 This behaviour can be changed by choosing other named method or by setting
-the collection naming routine with a C<closure> as exlained in L<Mongoose::Role::Naming>.
+the collection naming routine with a C<closure> as explained in L<Mongoose::Role::Naming>.
 
 =head1 REPOSITORY
 
@@ -311,29 +369,17 @@ Fork me on github: L<http://github.com/rodrigolive/mongoose>
 
 =head1 BUGS
 
-This is a WIP, now *beta* quality software.
-
-Report bugs via Github Issue reporting L<https://github.com/rodrigolive/mongoose/issues>.
+This is a WIP. Please report bugs via Github Issue reporting L<https://github.com/rodrigolive/mongoose/issues>.
 Test cases highly desired and appreciated.
 
 =head1 TODO
 
 * Better error control
 
-* Finish-up multiple database support
-
 * Allow query->fields to control which fields get expanded into the object.
 
-* Cleanup internals.
-
-* More tests and use cases.
-
 * Better documentation.
 
-=head1 SEE ALSO
-
-L<KiokuDB>
-
 =head1 AUTHOR
 
     Rodrigo de Oliveira (rodrigolive), C<rodrigolive@gmail.com>
@@ -351,6 +397,7 @@ L<KiokuDB>
     Allan Whiteford (allanwhiteford)
     Kartik Thakore (kthakore)
     David Golden (dagolden)
+    Mohammad S Anwar (manwar)
 
 =head1 LICENSE
 
